#!/bin/env -S python

from matplotlib import pyplot, patheffects
from matplotlib.axes import Axes
from adjustText import adjust_text


from data import (pay, release)


def center(data: list[tuple[str, list[float]]]):
    for axis_index in range(len(data[0][1])):
        min_ = min(data, key=lambda e: e[1][axis_index])[1][axis_index]
        max_ = max(data, key=lambda e: e[1][axis_index])[1][axis_index]
        delta = max_ - min_
        for (_, values) in data:
            original = values[axis_index]
            value = (original - min_) / delta
            value = (value * 2) - 1
            values[axis_index] = value


# from some image, online
to_rank = {
    'c#',
    'assembly',
    'bash',
    'basic',
    'brainfuck',
    'c',
    'c++',
    'cobol',
    'css',
    'dart',
    'elixir',
    'excel',
    'fim++',
    'fortran',
    'go',
    'haskell',
    'java',
    'javascript',
    'jsfuck',
    'julia',
    'kotlin',
    'lisp',
    'lolcode',
    'lua',
    'matlab',
    'objective-c',
    'ocaml',
    'perl',
    'php',
    'punch cards',
    'python',
    'r',
    'redstone',
    'ruby',
    'rust',
    'sass',
    'scala',
    'scratch',
    'sql',
    'swift',
    'typescript',
    'vb',
    'wasm',
    'whitespace',
    'zsh',
}


ranked = list[tuple[str, list[float]]](
    (label, [pay[label], release[label]])
    for label in to_rank
    if label in pay and label in release
)
center(ranked)


print("\nunused", [l for l in pay.keys() if not l in map(lambda e: e[0], ranked)])
print("\nunused", [l for l in release.keys() if not l in map(lambda e: e[0], ranked)])
print("\nnot found", [l for l in to_rank if not l in map(lambda e: e[0], ranked)])
print("\nranked", ranked)


(figure, axes) = pyplot.subplots()
assert type(axes) is Axes
axes.axhline(y=0, color='k')
axes.axvline(x=0, color='k')
axes.set_xlim(-1, 1)
axes.set_ylim(-1, 1)
axes.axis("off")
drop_white = patheffects.withStroke(linewidth=3,foreground="white")
texts = [axes.text(p, r, l, path_effects=[drop_white]) for (l, [r, p]) in ranked]
adjust_text(texts, only_move={'points':'y', 'texts':'y'})


axes.text(1.1, 0, 'connait\nle sexe', path_effects=[drop_white])
axes.text(-1.3, 0, 'ne connait\npas le sexe', path_effects=[drop_white])
axes.text(0, 1.1, 'a le\nsexe', path_effects=[drop_white])
axes.text(0, -1.2, 'n\'a pas\nle sexe', path_effects=[drop_white])

pyplot.show()
